const request = require('request-promise-native')
const parse = require('./models/parse')

class FinesseClient {
  constructor({host, username, password}) {
    this.host = host
    this.url = `https://${host}/finesse/api`
    this.username = username
    this.password = password
    this.auth = {
      username,
      password
    }
  }

  async getUser(id) {
    try {
      const options = {
        // json: true,
        'auth': {
          'user': this.username,
          'pass': this.password,
          'sendImmediately': true
        }
      }
      const response = await request(`${this.url}/User/${id}`, options)
      const json = parse.xml2js(response)
      if (json.User) {
        // has data
        return json.User
      } else {
        throw 'No user found'
      }
    } catch (e) {
      throw e
    }
  }

  async get(type, id, query) {
    try {
      const options = {
        // json: true,
        'auth': {
          'user': this.username,
          'pass': this.password,
          'sendImmediately': true
        },
        qs: query
      }
      const response = await request(`${this.url}/${type}/${id}`, options)
      const json = parse.xml2js(response)
      // console.log('json', json)
      if (json[type]) {
        // good response
        return json[type]
      } else {
        throw 'Not found'
      }
    } catch (e) {
      throw e
    }
  }

  async list(type, query) {
    try {
      const options = {
        // json: true,
        'auth': {
          'user': this.username,
          'pass': this.password,
          'sendImmediately': true
        },
        qs: query
      }
      const response = await request(`${this.url}/${type}s`, options)
      const json = parse.xml2js(response)
      if (json[type + 's']) {
        // good response
        const data = json[type + 's'][type]
        if (data) {
          if (Array.isArray(data)) {
            return data
          } else {
            // not array - return  data inside an array
            return [data]
          }
        } else {
          // no data - return empty array
          return []
        }
      } else {
        throw 'Not found'
      }
    } catch (e) {
      throw e
    }
  }

  async listFromTeam(teamId, type, query) {
    try {
      const options = {
        // json: true,
        'auth': {
          'user': this.username,
          'pass': this.password,
          'sendImmediately': true
        },
        qs: query
      }
      const response = await request(`${this.url}/Team/${teamId}/${type}s`, options)
      const json = parse.xml2js(response)
      if (json[type + 's']) {
        // good response
        const data = json[type + 's'][type]
        if (data) {
          if (Array.isArray(data)) {
            return data
          } else {
            // not array - return  data inside an array
            return [data]
          }
        } else {
          // no data - return empty array
          return []
        }
      } else {
        throw 'Not found'
      }
    } catch (e) {
      throw e
    }
  }

  async saveToTeam(teamId, type, data, query) {
    try {
      // const xmlData = js2xmlparser.parse(type + 's', data)
      const xmlData = parse.js2xml(data)
      // console.log('xmlData', xmlData)
      const options = {
        // json: true,
        'auth': {
          'user': this.username,
          'pass': this.password,
          'sendImmediately': true
        },
        headers: {
          'content-type': 'application/xml;charset=utf-8'
        },
        body: xmlData,
        qs: query
      }
      return request.put(`${this.url}/Team/${teamId}/${type}`, options)
    } catch (e) {
      throw e
    }
  }

  async getFromTeam(teamId, type) {
    try {
      const options = {
        // json: true,
        'auth': {
          'user': this.username,
          'pass': this.password,
          'sendImmediately': true
        }
      }
      const response = await request(`${this.url}/Team/${teamId}/${type}`, options)
      return parse.xml2js(response)
    } catch (e) {
      throw e
    }
  }
}

module.exports = FinesseClient
